'''
/** Original work Copyright 2024 damiano IU3QGD
 * https://to-be-defined
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.
 */
'''
# =================================================================================
# the idea is that this produces a Frame with all the possible commands in it
# Since python need the parent when building the gui, this must be provided
# Remember that the caller is responsible to layout this one

from __future__ import annotations

from queue import Queue
from threading import Thread
import threading
from tkinter import  IntVar

from debug_gui import LogPanel
from ipp_parser import Qsk5_req_identity, Qsk5_req_set_radio_params,\
    Qsk5_req_scan_loud,\
    Qsk5_req_syscmd
import tkinter.ttk as ttk


class AppMainGui(ttk.Frame):

    # -----------------------------------------------------------------------------
    # Constructor, note that this component layout is done by the caller
    def __init__(self, container, stat : 'Qpystat' ):
        ttk.Frame.__init__(self, container, borderwidth=3, relief='ridge')     
        
        self.stat=stat
    
        self._newButtons_aPanel(self).pack(fill='x')
        self._newButtons_bPanel(self).pack(fill='x')
        self._newSpecialReqPanel(self).pack(fill='x')
        self._newConfigRadioPanel(self).pack(fill='x')
        self._newScanLoudestPanel(self).pack(fill='x')
        
        self.cmdlog = LogPanel(self.stat,self,"Commands Log")
        self.cmdlog.pack(fill='both',expand=True)
        
        self.println("Command init complete")


    # -------------------------------------------------------------------------------
    # make a new frame and put input line in it
    def _newConfigRadioPanel(self, parent):

        panel=ttk.Frame(parent,borderwidth=3, relief='groove')
        
        ttk.Label(panel,text="rx tx Frequency (x10): ").pack(side='left')
        
        # this variable will store whatever the user has typed
        self.rxtxFrequency = IntVar(panel,value=44611000)
        
        ttk.Entry(panel, textvariable=self.rxtxFrequency).pack(side='left')

        ttk.Label(panel,text="  Power bias: ").pack(side='left')
        
        # this variable will store whatever the user has typed
        self.txPowerBias = IntVar(panel,value=10)
        
        ttk.Entry(panel, textvariable=self.txPowerBias, width=4).pack(side='left')

        ttk.Button(panel,text="Set Parameters" , command=self._reqSetRadioParams ).pack(side='left',padx=5)
        
        return panel

    # -------------------------------------------------------------------------------
    # make a new frame and put input line in it
    def _newSpecialReqPanel(self, parent):

        panel=ttk.Frame(parent,borderwidth=3, relief='groove')
        
        ttk.Label(panel,text="Param0 ").pack(side='left')
        
        # this variable will store whatever the user has typed
        self._specreqP0 = IntVar(panel,value=0)
        
        ttk.Entry(panel, textvariable=self._specreqP0).pack(side='left')

        ttk.Label(panel,text="  Param1 ").pack(side='left')
        
        # this variable will store whatever the user has typed
        self._specreqP1 = IntVar(panel,value=0)
        
        ttk.Entry(panel, textvariable=self._specreqP1, width=4).pack(side='left')

        ttk.Button(panel,text="Special Request" , command=self._reqSpecial ).pack(side='left',padx=5)
        
        return panel

    # --------------------------------------------------------------------------------------
    def _reqSpecial(self):
        v0=self._specreqP0.get();
        v1=self._specreqP1.get();
        cmd = Qsk5_req_syscmd(3,v0,v1)
        self.stat.qpypoll.queuePut(cmd)
    
    
    
    # --------------------------------------------------------------------------------------
    def _reqSetRadioParams(self):
        try:
            rxfreq1x = int(self.rxtxFrequency.get())
        except Exception as _exc:
            rxfreq1x=0
            
        try:
            powerb = int(self.txPowerBias.get())
        except Exception as _exc:
            powerb=0

        cmd = Qsk5_req_set_radio_params(rxfreq1x,powerb,0)
        self.stat.qpypoll.queuePut(cmd)
        
        self.stat.app_println("qant set f="+str(cmd))


    # -------------------------------------------------------------------------------
    # make a new frame and put input line in it
    def _newScanLoudestPanel(self, parent):

        panel=ttk.Frame(parent,borderwidth=3, relief='groove')
        
        ttk.Label(panel,text="Center frequency (x10): ").pack(side='left')
        
        # this variable will store whatever the user has typed
        self.scanLoudFrequency = IntVar(panel,value=14400000)
        
        ttk.Entry(panel, textvariable=self.scanLoudFrequency).pack(side='left')

        ttk.Label(panel,text="  Scan Time (cs): ").pack(side='left')
        
        # this variable will store whatever the user has typed
        self.scanTime_cs = IntVar(panel,value=50)
        
        ttk.Entry(panel, textvariable=self.scanTime_cs, width=5).pack(side='left')

        ttk.Button(panel,text="SCAN loudest F" , command=self._reqScanLoud ).pack(side='left',padx=5)
        
        return panel

    # --------------------------------------------------------------------------------------
    def _reqScanLoud(self):
        try:
            a_f = int(self.scanLoudFrequency.get())
        except Exception as _exc:
            a_f=44611000
            
        try:
            a_t = int(self.scanTime_cs.get())
        except Exception as _exc:
            a_t=20

        cmd = Qsk5_req_scan_loud(a_t,a_f)
        self.stat.qpypoll.queuePut(cmd)
        
        self.stat.app_println("request="+str(cmd))


    # -------------------------------------------------------------------------------
    # make a new frame and put buttons into it, but stay in this class
    def _newButtons_aPanel(self, container):
        cframe = ttk.Frame(container, borderwidth=3, relief='flat')

        ttk.Button(cframe,text="Polling" , command=self._raiseQpypoll ).pack(side='left')
        ttk.Button(cframe,text="Messenger" , command=self._raiseMessenger ).pack(side='left')
        ttk.Button(cframe,text="DTMF" , command=self._raiseDtmf ).pack(side='left')
        ttk.Button(cframe,text="EEprom" , command=self._raiseEeprom ).pack(side='left')
        ttk.Button(cframe,text="Debug" , command=self._raiseDebug ).pack(side='left')
        ttk.Button(cframe,text="Spectrum" , command=self._raiseSpectrum ).pack(side='left')

        return cframe
    
    # --------------------------------------------------------------------------------------
    def _raiseQpypoll(self):
        self.stat.qpypoll.deiconify()

    # --------------------------------------------------------------------------------------
    def _raiseMessenger(self):
        self.stat.qsmsg.deiconify()

    # --------------------------------------------------------------------------------------
    def _raiseDtmf(self):
        self.stat.qsdtmf.deiconify()

    # --------------------------------------------------------------------------------------
    def _raiseDebug(self):
        self.stat.dbgls.deiconify()

    # --------------------------------------------------------------------------------------
    def _raiseEeprom(self):
        self.stat.qseeprom.deiconify()

    # --------------------------------------------------------------------------------------
    def _raiseSpectrum(self):
        self.stat.spectrum.deiconify()

    # -------------------------------------------------------------------------------
    def _newButtons_bPanel(self, container):
        cframe = ttk.Frame(container, borderwidth=3, relief='flat')

        ttk.Button(cframe,text="Req Identity" , command=self._reqRadioIdentity ).pack(side='left')
        ttk.Button(cframe,text="Tasto M" , command=self._sendKey_Menu ).pack(side='left')
        ttk.Button(cframe,text="PTT click" , command=self._sendKey_PTT_click ).pack(side='left')
        ttk.Button(cframe,text="PTT release" , command=self._sendKey_PTT_release ).pack(side='left')
        ttk.Button(cframe,text="Reboot" , command=self._reqRadioReboot ).pack(side='left')

        return cframe
    
    # --------------------------------------------------------------------------------------
    def _reqRadioReboot(self):
        cmd = Qsk5_req_syscmd(1)
        self.stat.qpypoll.queuePut(cmd)
    
    # --------------------------------------------------------------------------------------
    def _sendKey_Menu(self):
        key_event_menu = 0b00100000 | 4;
        cmd = Qsk5_req_set_radio_params(0,0,key_event_menu)
        self.stat.qpypoll.queuePut(cmd)

    # --------------------------------------------------------------------------------------
    def _sendKey_PTT_click(self):
        key_event_menu = 0b00100000 | 1;
        cmd = Qsk5_req_set_radio_params(0,0,key_event_menu)
        self.stat.qpypoll.queuePut(cmd)

    # --------------------------------------------------------------------------------------
    def _sendKey_PTT_release(self):
        key_event_menu = 0b01000000 | 1;
        cmd = Qsk5_req_set_radio_params(0,0,key_event_menu)
        self.stat.qpypoll.queuePut(cmd)
    
    # --------------------------------------------------------------------------------------
    def _reqRadioIdentity(self):
        cmd = Qsk5_req_identity()
        self.stat.qpypoll.queuePut(cmd)

    # ---------------------------------------------------
    def println(self, msg):
        self.cmdlog.println(msg)

# -------------------------------------------------------------------------
# I can both type commands from console or use an event or a button
# so, there has to be a class that manage the queue of incoming commands
class InputParserThread (Thread):

    # ----------------------------------------------------------------
    def __init__(self, stat : 'Qpystat'):
        Thread.__init__(self,name="Input Parser",daemon=True)
        
        self.stat = stat
        
        self.oqueue = Queue()
    
    # ------------------------------------------------------
    # use this method to queue a command to be executed
    def queuePut (self,line):
        self.oqueue.put(line)
    
    # --------------------------------------------------------------
    # Called by the Thread, do NOT call it, use start()
    def run(self):
        print ("START Thread "+self.name)
        
        while self._parseOneLine_A():
            pass
            
        # this has to be the last one
        self.stat.appguithread.endGuiConsole()
        
        print ("END Thread ", self.name)

    # ---------------------------------------------------------------
    # _parseOneLine_B one input line and return False when need to terminate
    def _parseOneLine_A(self)-> bool:

        inline = self.oqueue.get()

        self.log_println("Request: "+inline)

        if inline == "quit":
            return False

        try:
            # need to protect from exceptions
            self._parseOneLine_B(inline)
        except Exception as _exc:
            self.log_println("parseOneCommand: "+str(_exc))
            
        return True



            
    # ---------------------------------------------------------------------
    # This is for when a command is called by the command line
    def _parseOneLine_B(self, line : str ):

        if line.startswith('show stat'):
            self.log_println(self.stat)
            
        elif line == 'threads':
            self.log_println("Threads registered")
            self.log_println("  isAlive\t isDaemon\t thread_name")

            for thread in threading.enumerate(): 
                self.log_println("  "+str(thread.is_alive())+"\t"+str(thread.daemon)+"  \t"+thread.name)

        elif line == 'clients':
            self.log_println("Clients registered")

        else:
            self.log_println("unsupported command")
            
            
    # ---------------------------------------------------------------------
    # To make code cleaner and shorter
    def log_println(self, msg):
        self.stat.appmaingui.println(msg)





        

