
from __future__ import annotations

from pickle import TRUE
import struct
from tkinter import ttk
import tkinter

from glob_gui import TableColdef, EEblockGui


# -------------------------- NOTES
# See https://realpython.com/iterate-through-dictionary-python/
ecol_idx="rub_idx"
ecol_band="fmr_band"
ecol_chipchan="rub_chipchan"
ecol_freq="rub_freq"

    
# -------------------------------------------------------
# Generic way to define a map from a column name to some properties        
cols_map = { 
    ecol_idx      : TableColdef(ecol_idx,'IDx',50), 
    ecol_band     : TableColdef(ecol_band,"Band",50),
    ecol_chipchan : TableColdef(ecol_chipchan,"C Channel",80),
    ecol_freq     : TableColdef(ecol_freq,"Frequency",100),
    }


# ===================================================================
# It is safer to deal with objects instead of a generic map
class FmradioRow:

    '''
    struct FM_EE_chan_t
        {
        uint16_t fmchip_chan_po :10;    // reg03 there are 10 bits from 0 to 1023 included, but 0 is NULL !
        uint16_t fmchip_space   :2;     // reg05 bits 4:5
        uint16_t fmchip_band    :2;     // reg05 bits 6:7
        uint16_t free           :2;     // MSB on LE CPU
        }    
    '''
    
    def getBandFreqBase(self):
        if self.fmr_band == 0:
            return 875
        elif self.fmr_band == 1:
            return 760
        elif self.fmr_band == 2:
            return 760
        elif self.fmr_band == 3:
            return 640
        else:
            return 0
    
    
    # ----------------------------------------------------------------
    # Constructor, the parameter is a dictionary holding the actual data received from the radio
    # this will receive a short that will need to be decoded
    def __init__(self, index : int, raw_val : int ):
        self.fmr_idx=index
        self.fmr_band= (raw_val >> 12) & 0b11
        
        chipchan_po=(raw_val & 0b1111111111)
        
        if chipchan_po == 0 or chipchan_po == 0b1111111111:
            self.fmr_chipchan=''
            self.fmr_freq=''
        else:
            self.fmr_chipchan=chipchan_po -1
            fmbase=self.getBandFreqBase()
            
            fmr_freq=fmbase+self.fmr_chipchan
            
            self.fmr_freq = "%d.%d Mhz" % (fmr_freq/10, fmr_freq%10)
        
        self.show=True
            
    # -----------------------------------------------------------------
    # return the list of values to show into the table, the GUI table
    # NOTE that the order has to be consistent with the GUI definition
    def getValuesForTreeview(self):
        risul = (
            self.fmr_idx,
            self.fmr_band,
            self.fmr_chipchan,
            self.fmr_freq
            )
            
        return risul         
    
    
# ========================================================================
# Let me try to have some decent OO wrapping in this shitty language
class FmradioList(list[FmradioRow]):

    # -----------------------------------------------------------------
    def setListFromPush (self, short_array : [int]):
        
        self.clear()  # clear current content

        print('short array',short_array)
        
        for index,row in enumerate(short_array):
            self.append(FmradioRow(index, row))
    
    # ------------------------------------------------------------------
    def getExtensionUsingNumber(self, extnum : str ) -> FmradioRow :
        
        for i_row in self:
            arow : FmradioRow = i_row
            if extnum == arow.rub_id:
                return arow
    
        return None
    
            
    # -----------------------------------------------------------------
    # go trough the list and decide if to show anything
    def decideShow(self,filterval : str, hide_unavilable : bool ):
        
        for row in self:
            row.decideShow(filterval, hide_unavilable)

    
    
    








# =====================================================================
# the table with the list of FM channels
class EEblock_0x0E40_FMtable_Gui (EEblockGui):

    ee_chan_max=20

    # ----------------------------------------------------------------
    def __init__(self, stat : 'Qpystat', parent_frame : ttk.Frame ):
        EEblockGui.__init__(self, parent_frame, 0x0E40, 2 * self.ee_chan_max )
        
        self.stat = stat
        
        self.fmradiolist = FmradioList()
        
        self.c_panel=self._newCenterPanel(self)
        
        self.c_panel.pack(fill='both', expand=True)
        
    # ------------------------------------------------------------------------
    def setListFromPush(self, extensionlist):
        self.toplevel.after_idle(self._setListFromPushGui,extensionlist)
        
    # ---------------------------------------------------------------------------
    # This MUST be used within a GUI thread !!!
    def _updateGuiSwing(self):

        short_array = struct.unpack('<20H',self.ee_block.ee_bytearray)
        
        self.fmradiolist.setListFromPush(short_array)
        
        self._filterEntries()


    # -------------------------------------------------------------------
    # At any time it should be possible to filter the entries and show them
    # Note that this may be called at the beginning , when there is nothing in the list...
    def _filterEntries(self ):    
        
        for a_row in self.fmradiolist:
            e_row : FmradioRow = a_row
            
            if e_row.show:
                self._ensureVisible(e_row)
            elif self.etbl.exists(e_row.extnum):
                self.etbl.detach(e_row.extnum)


    # -----------------------------------------------------------------
    # There is no write, at the moment
    def updateBlockFromGui(self) -> bool:
        return False
        
    # ------------------------------------------------------------------------
    # Utility, to make code cleaner
    # NOTE that part of the magic works because the extension number is a unique key
    # This MUST be run from swing thread
    def _ensureVisible(self, row : FmradioRow ):

        if self.etbl.exists(row.fmr_idx):
            self.etbl.reattach(row.fmr_idx,'',0)
        else:
            self.etbl.insert(
                parent='',
                index='end',
                iid=row.fmr_idx,    
                values=row.getValuesForTreeview()
                )

    # -------------------------------------------------------------------------------
    # On center panel there is the table and scroll bar
    # NOTE: The layout of this is described by caller
    # NOTE: The style for this part is defined in the main gui, using style
    def _newCenterPanel(self, parent):
        apanel = ttk.Frame( parent, borderwidth=3, relief='ridge')

        apanel.grid_rowconfigure(0, weight = 1)
        apanel.grid_columnconfigure(0, weight = 1)

        
        # list(cols_map.keys()) is needed since just cols_map.keys() produce a view of the map and the Treeview does not like it
        # takefocus=0 does not work to prevent the tree to steal focus from the Entry on tree filter
        self.etbl = ttk.Treeview(apanel,columns=list(cols_map.keys()), show='headings')
                
        for colu in cols_map.values():
            self.etbl.heading(colu.col_name, text=colu.col_label, anchor='w',command=lambda _col=colu.col_name : self._treeview_sort_column( _col ))
            self.etbl.column(colu.col_name, minwidth=50, width=colu.width, stretch=colu.stretch)

        # an item as example        
        #self.etbl.insert(parent='',index='end',iid=0,text='',values=('1','band','chan','88.88'))

        self.etbl.grid(row=0, column=0, sticky='nsew')
        
        scrollbar = ttk.Scrollbar(apanel, orient=tkinter.VERTICAL, command=self.etbl.yview)
        self.etbl.configure(yscroll=scrollbar.set)
        
        scrollbar.grid(row=0, column=1, sticky='ns')
        
        return apanel

    # -------------------------------------------------------------------
    # Used to sort a table by a given column
    # it gets all children keys, create a list that gets sorted and then move items around        
    def _treeview_sort_column(self, tblcol_name ):
        col = cols_map[tblcol_name]
        
        # the following returns a tuple of all "keys", eg: ('I001', 'I002', 'I003', 'I004', 'I005'...... 
#        print(self.etbl.get_children())

        lista = [(self.etbl.set(key, tblcol_name), key) for key in self.etbl.get_children()]
        lista.sort(reverse=col.sort_desc)
    
        # reverse the sorting, for next time
        col.sort_desc = not col.sort_desc 
    
        # rearrange items in sorted positions. enumerate returns the index and the actual value
        for index, (_val, key) in enumerate(lista):
            self.etbl.move(key, '', index)

    # ----------------------------------------------------------------------
    # Clear the whole table    
    def clear(self):
        self.etbl.focus_set()

        children = self.etbl.get_children()

        for child in children:
            self.etbl.delete(child)


