'''
/** Original work Copyright 2024 damiano IU3QGD
 * https://to-be-defined
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.
 */

# the basic IPP chunk is a chunk id and a length
# NOTE that when the id is ODD the length is 16 bits and therefore there is a dummy char after the id

# this is the equivalent of toString()
    def __str__(self):

'''


from __future__ import annotations

import struct


from ipp_chunkid import IPPU_pollprintf_resid, IPPU_pollradiostat_resid, \
    IPPU_pollqsmsg_resid, IPPU_ident_resid, IPPU_radiocfg_resid, \
    IPPU_generic_resid, IPPU_scan_loudest_resid, IPPU_polldtmf16_resid, \
    IPPU_res_counters16, IPPU_res_spectrum16, IPPU_ident_reqid, \
    IPPU_req_spectrum, IPPU_sendqsmsg_reqid, IPPU_radiocfg_reqid, \
    IPPU_senddtmf16_reqid, IPPU_scan_loudest_reqid, \
    IPPU_systemcmd_reqid, IPPU_req_eeprom, IPPU_res_eeprom
from uvk5_lib import Libuvk5serial


# ==========================================================================
# I need a generic block of memory, to send data around
# this will be used for both read and write
class EEblock ():

    # -----------------------------------------------------------------------
    def __init__(self, ee_start_a : int, ee_len: int,  ee_bytearray : bytearray ):
        self.ee_start_a=ee_start_a
        self.ee_len = ee_len
        # because crap python does not really know what is incoming
        self.ee_bytearray=bytearray(ee_bytearray)

    # -------------------------------------------------
    # return the block actual data size or zero if no data available
    def get_data_len(self):
        if self.ee_bytearray:
            return len(self.ee_bytearray)
        else:
            return 0
        
    
    
    # -------------------------------------------------
    # return the byte at given index
    # note that it will return 0 if there is no buffer !
    def get_byte(self, byte_index : int ) -> int:
        if self.ee_bytearray and len(self.ee_bytearray) > byte_index:
            return self.ee_bytearray[byte_index];
        else:
            return 0

    # -------------------------------------------------
    # no, there is no byte type, everything is an int until you cannot push it anymore
    # plain stupid, ah, I know, pass an array ob bytes of len 1.... do you also want a pine cone in your ass with it ?
    def set_byte(self, byte_index : int, byte_value : int ):
        if self.ee_bytearray and len(self.ee_bytearray) > byte_index:
            self.ee_bytearray[byte_index] = byte_value;
        else:
            print('EEblock  invalid set_byte inded ',byte_index)

    
    # -------------------------------------------------
    # return a string
    def get_string(self, start=None, end=None):
        if not self.ee_bytearray:
            return ''

        return self.ee_bytearray.decode('ascii','replace')
        
    # -------------------------------------------------
    # set the array as a string
    def set_string(self, astring : str ):
        decoded=astring.encode('ascii',errors='ignore')
        
        l_decoded=len(decoded)
        l_barray=len(self.ee_bytearray)
        
        if l_decoded == l_barray:
            self.ee_bytearray = bytearray(decoded)
        elif l_decoded < len(self.ee_bytearray):
            self.ee_bytearray[0:l_decoded] = decoded
        else:    
            self.ee_bytearray = bytearray(decoded[0:l_barray])
            
        #print(self.ee_bytearray)



    # --------------------------------------------------
    # this is the equivalent of toString()
    def __str__(self):
        return str(self.ee_start_a)+' '+str(self.ee_len)




# ===================================================================================
# Basic class
class IppChunk:
    
    # -------------------------------------------------------------------------------
    # standard constructor
    def __init__(self,chunk_id, chunk_len, rxbuffer):
        self.chunk_id=chunk_id
        self.chunk_len=chunk_len
        self.rxbuffer=rxbuffer
        self.remain_len=len(rxbuffer)

    # --------------------------------------------------
    # this is the equivalent of toString()
    def __str__(self):
        return "id="+str(self.chunk_id)+" len="+str(self.chunk_len)

# ===================================================================================
# you can "decode" into this one
'''
struct IPPU_radio_status_res
   {
   enum IPPU_cid chunk_id;  // IPPU_radiostat_resid
   uint8_t  chunk_len;
   uint16_t rx_rssi;
   uint32_t rx_freq_1x;   // current RX frequency
   uint32_t tx_freq_1x;   // current TX frequency
   uint16_t rx_noise;
   uint16_t rx_glitch;
   uint16_t batteryV_2dec;   // it is reported as a 2 dec fixed
   uint8_t  tx_power;        // current TX power
   uint8_t  padding;
   union Rstate radio_status;
   uint16_t tc_cpu_2dec;
   };
'''
class Qsk5_res_RadioStatus(IppChunk):

    # -------------------------------------------------------------------------------
    # note that the given buffer INCLUDES the id and len
    def __init__(self,chunk : IppChunk ):
        IppChunk.__init__(self,chunk.chunk_id,chunk.chunk_len,chunk.rxbuffer)

        rx_rssi,rxfreq,txfreq,rx_noise,rx_glitch,batteryV_2dec,txpwr,_a,radio_status,tc_cpu_2dec=struct.unpack_from('<HIIHHHBBHH',self.rxbuffer,2)

        self.rx_rssi=rx_rssi
        self.rxfreq=rxfreq
        self.txfreq=txfreq
        self.rx_noise=rx_noise
        self.rx_glitch=rx_glitch
        self.batteryV_2dec=batteryV_2dec
        self.batteryV = float(batteryV_2dec)/100.0
        self.txpwr=txpwr
        self.radio_status=radio_status
        self.tc_cpu_2dec=tc_cpu_2dec;
        self.tc_cpu=float(tc_cpu_2dec)/100.0;
        


# ===================================================================================
# you can "decode" into this one
'''
enum IPPU_cid chunk_id;   // IPPU_ident_resid
uint8_t chunk_len;        // sizeof(struct IPPU_ident_res)
char hw_model[12];        // UV-K5
char hw_version[12];      // possibly empty
char sw_model[12];        // damiano
char sw_version[14];      // 2024071711
uint32_t hw_cpuid[4];     // as taken from CPU
'''
class Qsk5_res_RadioIdentity(IppChunk):

    # -------------------------------------------------------------------------------
    # note that the given buffer INCLUDES the id and len
    # will return an array of elements as indicated in the decode
    # NOTE that it all begins at 2
    # NOTE that 4L result in four longs
    def __init__(self,chunk : IppChunk ):
        IppChunk.__init__(self,chunk.chunk_id,chunk.chunk_len,chunk.rxbuffer)

        # AHHHHHHHH, can whoever invented python go to HELL
        # you NEED the < at the beginning, otherwise it will just get the NUMBER of bytes to convert wrong
        # yes, the NUMBER OF BYTES, I know, the value is fair game, but the NUMBER of bytes ? idiots
        hw_model,hw_version,sw_model,sw_version,id0,id1,id2,id3=struct.unpack_from('<12s12s12s14s4I',self.rxbuffer,2)
        
        self.hw_model=hw_model.decode('ascii','replace')
        self.hw_version=hw_version.decode('ascii','replace')
        self.sw_model=sw_model.decode('ascii','replace')
        self.sw_version=sw_version.decode('ascii','replace')

        self.id0=id0
        self.id1=id1
        self.id2=id2
        self.id3=id3

    # --------------------------------------------------
    def __str__(self):
        return self.hw_model+" "+self.sw_version+" "+str(self.id0)


# ===================================================================================
'''
struct IPPU_radiocfg_res
    {
    enum IPPU_cid chunk_id;      // IPPU_radiocfg_resid
    uint8_t  chunk_len;
    uint16_t pad_even;
    uint32_t rx_freq_1x;       // receive on this f, zero means leave it alone _1x means that you need to mult by 10
    uint32_t tx_freq_1x;       // send on this f, zero means leave it alone _1x means that you need to mult by 10
    uint8_t  tx_power;           // 0 means do not touch config, FF max power, intermediate numbers as appropriate
    uint8_t  pad2[3];
    };
'''
class Qsk5_res_RadioConfiguration(IppChunk):

    # -------------------------------------------------------------------------------
    # note that the given buffer INCLUDES the id and len
    # will return an array of elements as indicated in the decode
    # NOTE that it all begins at 2
    # NOTE that 4L result in four longs
    def __init__(self,chunk : IppChunk ):
        IppChunk.__init__(self,chunk.chunk_id,chunk.chunk_len,chunk.rxbuffer)

        _,rxfreq_1x,txfreq_1x,txpower,_,_,_=struct.unpack_from('<HIIBBBB',self.rxbuffer,2)

        self.rxfreq_1x=rxfreq_1x
        self.txfreq_1x=txfreq_1x
        self.txpower=txpower
        
    # --------------------------------------------------
    def __str__(self):
        return str(self.rxfreq_1x)


# ==================================================================================
'''
struct IPPU_generic_res
    {
    enum IPPU_cid chunk_id;       // IPPU_error_resid
    uint8_t chunk_len;            // sizeof(myself)
    uint16_t from_reqid;          // the original request this response is bound to
    uint16_t reason;              // some sort of reason for the message
    uint16_t rescode;             // some sort of error code
    char     message[40];         // a short message
    };
'''
class Qsk5_res_Generic(IppChunk):

    # --------------------------------------------------
    def __init__(self,chunk : IppChunk ):
        IppChunk.__init__(self,chunk.chunk_id,chunk.chunk_len,chunk.rxbuffer)

        from_reqid,reason,rescode,message=struct.unpack_from('<HHH40s',self.rxbuffer,2)

        self.from_reqid=from_reqid
        self.reason=reason
        self.rescode=rescode
        self.message=message.decode('ascii','replace')
        
    # --------------------------------------------------
    def __str__(self):
        return str(self.from_reqid)+" "+str(self.reason)+" "+str(self.rescode)+" "+self.message




# ==================================================================================
'''
struct IPPU_scan_loudest_res
    {
    enum IPPU_cid chunk_id;      // IPPU_scan_loudest_resid
    uint8_t  chunk_len;
    uint16_t pad_even;
    uint32_t rx_freq_1x;         // the found frequency
    // in future time I could return further info on CxCSS
    };
'''
class Qsk5_res_Loudest(IppChunk):

    # --------------------------------------------------
    def __init__(self,chunk : IppChunk ):
        IppChunk.__init__(self,chunk.chunk_id,chunk.chunk_len,chunk.rxbuffer)

        _,found_F_1x=struct.unpack_from('<HI',self.rxbuffer,2)

        self.found_F_1x=found_F_1x
        
    # --------------------------------------------------
    def __str__(self):
        return str(self.found_F_1x)

# ==================================================================================
'''
struct Uvk5Counters
    {
   uint16_t bfsk_rx_count;          // total number of received packets
   uint16_t bfsk_rx_crcbad_count;   // CRC bad received
   uint16_t bfsk_rx_ber_count;      // a sum of all ber errors count
   uint16_t bfsk_tx_count;          // total number of sent packets

   uint16_t rx_qsmsg_count;              // number of messages received
   uint16_t rx_qstxt_count;              // number of messages received
   uint16_t rx_qsack_count;              // number of ACK received
   uint16_t rx_ipp_count;                // number of possibly IPP packets

   uint8_t  bfsk_rx_badlen_count;   // packet is too big or too short
   uint8_t  bfsk_tx_radbusycount;   // radio was busy

    uint8_t  dtmf_ipp_rx_overflow_counter;
    uint8_t  dtmf_char_rx_overflow_counter;

    uint8_t  dtmf_rx_ack_counter;
    uint8_t  dtmf_rx_response_counter;
    uint8_t  dtmf_rx_call_counter;
    uint8_t  dtmf_tx_call_counter;   // TODO

    uint8_t EE_waddressbad_counter;  // write address is bad
    uint8_t free;
    };

/**
 * Counters are polled and this is what is returned
 * NOTE that I should map this packet INTo the TX serial buffer
 * NOTE that since this becomes part of an API, you can ONLY add pieces, NEVER delete !
 */
struct IPPU_res_counters
   {
    struct IPPU_header16 header;   // IPPU_res_counters16
    char   sw_version[12];         // the current fw version bound to the counters
    struct Uvk5Counters counters;  // the actual counters
   };

'''
class Qsk5_res_Counters(IppChunk):

    # --------------------------------------------------
    def __init__(self,chunk : IppChunk ):
        IppChunk.__init__(self,chunk.chunk_id,chunk.chunk_len,chunk.rxbuffer)

        sw_version,bfsk_rx_count,bfsk_rx_crcbad_count,bfsk_rx_ber_count,bfsk_tx_count=struct.unpack_from('<12sHHHH',self.rxbuffer,4)
        
        self.sw_version=sw_version.decode('ascii','replace')
        self.bfsk_rx_count=bfsk_rx_count
        self.bfsk_rx_crcbad_count=bfsk_rx_crcbad_count
        self.bfsk_rx_ber_count=bfsk_rx_ber_count
        self.bfsk_tx_count=bfsk_tx_count

        
    # --------------------------------------------------
    def __str__(self):
        return str(self.found_F_1x)

# =====================================================================
class Qsk5_res_Spectrum(IppChunk):
    '''
    struct IPPU_res_spectrum
       {
        struct IPPU_header16 header;  // IPPU_res_spectrum
       uint32_t start_freq_1x;       // the starting frequency
       uint32_t step_freq_1x;        // the step can be HUGE, but should really not be
       uint16_t samples_count;       // this is the number of steps (samples) to do
       uint16_t sample_time_cs;      // the time to take for a sample in cs, so you can do 100 samples per second, max
       uint8_t  samples[IPPU_SSAMPLES_MAX]; // from here onward there are the samples
       };
    '''

    # --------------------------------------------------
    def __init__(self,chunk : IppChunk ):
        IppChunk.__init__(self,chunk.chunk_id,chunk.chunk_len,chunk.rxbuffer)

        start_freq_1x, step_freq_1x, samples_count, sample_time_cs, samples=struct.unpack_from('<IIHH256s',self.rxbuffer,4)

        self.start_freq_1x=start_freq_1x
        self.step_freq_1x=step_freq_1x
        self.samples_count=samples_count
        self.sample_time_cs=sample_time_cs
        self.samples=samples

        
    # --------------------------------------------------
    def __str__(self):
        return str(self.start_freq_1x)



# =====================================================================

class Qsk5_res_Eeprom(IppChunk):
    '''
    struct IPPU_res_eeprom
       {
       struct IPPU_header8 header;  // IPPU_res_eeprom
       enum IPPU_EE_req_e ee_req;   // what request is it ?
       uint8_t  ee_data_len;        // the actual number of bytes, can be zero
       uint16_t ee_rw_address;      // read or write here
       uint16_t ee_rescode;         // response code to the request
       uint16_t padding;            // free for possible uses
       uint8_t  ee_data[128];       // max buffer of 128 bytes
       };
    
    '''
    # --------------------------------------------------
    def __init__(self,chunk : IppChunk ):
        IppChunk.__init__(self,chunk.chunk_id,chunk.chunk_len,chunk.rxbuffer)

        ee_req,ee_data_len,ee_rw_address,ee_rescode,_=struct.unpack_from('<BBHHH',self.rxbuffer,2)
        
        self.ee_req=ee_req
        self.ee_data_len=ee_data_len
        self.ee_rw_address=ee_rw_address
        self.ee_rescode_=ee_rescode

        self.ablock = bytearray()

        if self.ee_data_len > 0:
            self.ablock = EEblock(self.ee_rw_address, self.ee_data_len, self.rxbuffer[10:])
      
        
    # --------------------------------------------------
    def __str__(self):
        return 'Qsk5_res_Eeprom ee_address='+str(self.ee_rw_address)
























# ===================================================================================
# this is given the chunk, starting from the chunk id
class Qsk5_res_PollMessages(IppChunk):

    # -------------------------------------------------------------------------------
    # note that the given buffer INCLUDES the id and len
    def __init__(self,chunk : IppChunk ):
        IppChunk.__init__(self,chunk.chunk_id,chunk.chunk_len,chunk.rxbuffer)

        self.flags,self.from_uid,self.to_uid = struct.unpack_from('<BBB',self.rxbuffer,2)
        
        self.is_rx = self.flags & 0x01
        self.is_tx = self.flags & 0x02
        self.is_ack = self.flags & 0x04
        
        # starts on third byte
        self.messagestring = self.rxbuffer[6:].decode('ascii','replace') 

    # -----------------------------------------------------------------------
    def _get_StatusString(self):
        if self.is_ack:
            return "+"
        
        if self.is_tx:
            return ">"

        if self.is_rx:
            return "<"
        
        return "?"

    # --------------------------------------------------
    def __str__(self):
        return self._get_StatusString()+" "+str(self.from_uid)+" "+str(self.to_uid)+" "+self.messagestring

# ===================================================================================
# this is given the chunk, starting from the chunk id
class Qsk5_res_RadioPrintf(IppChunk):

    # -------------------------------------------------------------------------------
    # standard constructor
    def __init__(self,chunk : IppChunk ):
        IppChunk.__init__(self,chunk.chunk_id,chunk.chunk_len,chunk.rxbuffer)

        astring = self.rxbuffer[3:].decode('ascii','replace')
        
        self.print_this=astring

# ===================================================================================
'''
struct IPPU_dtmf_poll_res
    {
    enum IPPU_cid chunk_id;      // IPPU_polldtmf_resid
    uint8_t  chunk_len;          // whole chunk length
    uint8_t  isIpp;              // if true then the payload is an IPP
    uint8_t  align_even;         // Align even
    uint8_t  payload[0];         // I should map this into the send buffer
    };
'''
class Qsk5_res_PollDtmf(IppChunk):

    # -------------------------------------------------------------------------------
    # note that the given buffer INCLUDES the id and len
    # will return an array of elements as indicated in the decode
    # NOTE that it all begins at 2
    # NOTE that 4L result in four longs
    def __init__(self,chunk : IppChunk ):
        IppChunk.__init__(self,chunk.chunk_id,chunk.chunk_len,chunk.rxbuffer)

        # I know it is a chunk16
        is_ipp,_=struct.unpack_from('<BB',self.rxbuffer,4)
        
        self.is_ipp=is_ipp
        
        # use this if it is an ipp packet
        self.dtmf_bytes : bytearray = self.rxbuffer[6:]

        if self.is_ipp:
            self.dtmf_string = self.dtmf_bytes.hex()
        else:
            self.dtmf_string = self.dtmf_bytes.decode('ascii','replace')
        
    # --------------------------------------------------
    def __str__(self):
        return str(self.chunk_id)+' '+str(self.is_ipp)




# ===================================================================================
# You could immagine to wrap the GUI in the object.... possibly
# a parser is needed since a packet may have different chunks in it
# and ideally you need to unpack all of them
class Ippparser:
    
    # -------------------------------------------------------------------------------
    # standard constructor
    def __init__(self, rxbuffer : bytearray ):
        self.rxbuffer : bytearray = rxbuffer
        self.begin_index=0;
        self.remain_len=len(rxbuffer)

    # -------------------------------------------------------------------------------
    # there is a possible chunk to parse ?
    # a chunk is at least two bytes long chunk id and chunk len
    def hasNextChunk(self) -> bool:
        return self.rxbuffer and self.remain_len >= 2
            
    # -------------------------------------------------------------------------------
    # there can be more than one chunk in the answer
    # NOTE that now chunks can be of 8 or 16 bits length !
    # TODO, I could dispatch the chunks here !
    def getNextIppChunk(self) -> IppChunk:
        if not self.hasNextChunk():
            return None
        
        chunk_id=self.rxbuffer[self.begin_index]
        
        if chunk_id & 1 == 0:
            chunk_id,chunk_len=struct.unpack_from('<BB',self.rxbuffer,self.begin_index)
        else:
            chunk_id,_,chunk_len=struct.unpack_from('<BBH',self.rxbuffer,self.begin_index)
            
        if chunk_len < 1:
            # it happens if you mess up the header on packet creation
            print("getNextIppChunk: len=0 for id ",chunk_id)
            return None
        
        chunk=IppChunk(chunk_id, chunk_len, self.rxbuffer[self.begin_index:self.begin_index+chunk_len])
        
        self.remain_len = self.remain_len-chunk_len
        self.begin_index = self.begin_index+chunk_len
        
        if chunk_id == IPPU_pollprintf_resid:
            return Qsk5_res_RadioPrintf(chunk)
        elif chunk_id == IPPU_pollradiostat_resid:
            return Qsk5_res_RadioStatus(chunk)
        elif chunk_id == IPPU_pollqsmsg_resid:
            return Qsk5_res_PollMessages(chunk)
        elif chunk_id == IPPU_ident_resid:
            return Qsk5_res_RadioIdentity(chunk)
        elif chunk_id == IPPU_radiocfg_resid:
            return Qsk5_res_RadioConfiguration(chunk)
        elif chunk_id == IPPU_generic_resid:
            return Qsk5_res_Generic(chunk)
        elif chunk_id == IPPU_polldtmf16_resid:
            return Qsk5_res_PollDtmf(chunk)
        elif chunk_id == IPPU_scan_loudest_resid:
            return Qsk5_res_Loudest(chunk)
        elif chunk_id == IPPU_res_counters16:
            return Qsk5_res_Counters(chunk)
        elif chunk_id == IPPU_res_spectrum16:
            return Qsk5_res_Spectrum(chunk)
        elif chunk_id == IPPU_res_eeprom:
            return Qsk5_res_Eeprom(chunk)
        else:
            return chunk
         
         
         

# ===================================================================================
# commands to be executed will be sent to the poller using this as ancestor
# I need this since the serial channel is NON sharable
# well, not really, I could have a lock on an IPP request ....
# so, the whole "queue" issue disappear BUT you still need a separate thread, otherwise you freeze the GUI....
# worth it ? not really, since you would need a runner thread
# unless you create a runner thread each "request"..... I suppose a gen Z would do this.....
# Also, I am tempted to merge the GUI in here, but it will become a blob of code that becomes confusing 
# since this is just a data transfer...

class Qsk5_command:
    
    # -------------------------------------------------------------------------------
    # standard constructor requires the command identifier that is later used, if needed
    def __init__(self,command):
        self.command=command
        
    # -------------------------------------------------------------------------------
    # a command should be able to send a request trough the standard library
    # this MUST be overriden in the subclasses
    def send_request(self, serlib : Libuvk5serial ) -> Ippparser:
        return None




# ===================================================================================
# A command to read write eeprom
class Qsk5_req_eeprom(Qsk5_command):
    '''
    struct IPPU_req_eeprom
       {
       struct IPPU_header8 header;  // IPPU_req_eeprom
       enum IPPU_EE_req_e ee_req;   // what request is it ?
       uint8_t  ee_data_len;        // the actual number of bytes to read or write
       uint16_t ee_rw_address;      // read or write here
       uint16_t ee_read_pw;         // if required this is a possible read password
       uint16_t ee_write_pw;        // if required this is a possible write password
       uint8_t  ee_data[IPPU_EPPROM_DATALEN];   // max buffer of 128 bytes
       };
    '''
    # -------------------------------------------------------------------------------
    # If we have EE data then this is a write, otherwise it is a read
    def __init__(self, ee_address : int, ee_len, ee_data : bytearray ):
        Qsk5_command.__init__(self, 'eeprom_read_write')
        
        self.ee_address = ee_address
        self.ee_len  = ee_len;
        self.ee_data = ee_data

    # --------------------------------------------------
    # this is the equivalent of toString()
    def __str__(self):
        return self.command+' '+str(self.ee_address)

    # ---------------------------------------------------------------------------------------
    # See c header)
    def send_request(self, serlib : Libuvk5serial ) -> Ippparser:

        if  self.ee_data and len(self.ee_data) > 0:
            s_chunk = struct.pack('<BBHHH',2,self.ee_len,self.ee_address,0,0)+self.ee_data
            #print("EE write len ",self.ee_len)
        else:
            s_chunk = struct.pack('<BBHHH',1,self.ee_len,self.ee_address,0,0)
        
        buffer=serlib.ippp_send_chunk(IPPU_req_eeprom,s_chunk)
        
        return Ippparser(buffer)






























'''
# ===================================================================================
# A command to send system commands to the radio
class Qsk5_req_system(Qsk5_command):

        
    struct IPPU_req_system_cmd
       {
       struct IPPU_header8 header;  // IPPU_specialcmd_req
       enum IPPU_SYSCMD_e sys_cmd;  // the actual command
       uint8_t  padding;            // free for use
       uint16_t w_power;            // the power level I wish, 0=low 1=mid 2,high
       uint16_t paddingb;           // free to use
       uint32_t w_freq_1x;          // the frequency to apply
       };
    

    # -------------------------------------------------------------------------------
    # standard constructor
    def __init__(self, cmd_code : int ):
        Qsk5_command.__init__(self, 'system_cmd')
        
        self.cmd_code = cmd_code

    # --------------------------------------------------
    # this is the equivalent of toString()
    def __str__(self):
        return self.command+' '+str(self.cmd_code)

    # ---------------------------------------------------------------------------------------
    # See c header)
    def send_request(self, serlib : Libuvk5serial ) -> Ippparser:
        s_chunk = struct.pack('<BBHHI',self.cmd_code,0,0,0,0)
        
        buffer=serlib.ippp_send_chunk(IPPU_systemcmd_reqid,s_chunk)
        
        return Ippparser(buffer)

'''























# ===================================================================================
# this wraps a command to request a scan for the loudest frequency around center freq
class Qsk5_req_scan_loud(Qsk5_command):

    # -------------------------------------------------------------------------------
    # standard constructor
    def __init__(self, tmout_cs : int, center_F_1x : int ):
        Qsk5_command.__init__(self, "scan loudest")
        
        self.tmout_cs = tmout_cs
        self.center_F_1x = center_F_1x

    # --------------------------------------------------
    # this is the equivalent of toString()
    def __str__(self):
        return self.command+' '+str(self.tmout_cs)


    # ---------------------------------------------------------------------------------------
    # 
    '''
    struct IPPU_scan_loudest_req
        {
        enum IPPU_cid chunk_id;      // IPPU_scan_loudest_reqid
        uint8_t  chunk_len;
        uint8_t  scan_time_cs;       // from 20 to 160 cs
        uint8_t  scan_flags;
        uint32_t scan_center_freq;   // anything below VHF_UHHF_THRESHOLD_1X enables VHF filtering
        };
    '''
    def send_request(self, serlib : Libuvk5serial ) -> Ippparser:

        s_chunk = struct.pack('<BBI',self.tmout_cs,0,self.center_F_1x)
        
        buffer=serlib.ippp_send_chunk(IPPU_scan_loudest_reqid,s_chunk)
        
        return Ippparser(buffer)















# ===================================================================================
# this wraps a command to send a DTMF sequence
# note that the content must already be prepared as required
class Qsk5_req_send_dtmf(Qsk5_command):

    # -------------------------------------------------------------------------------
    # standard constructor
    def __init__(self,play_len_cs : int, play_mute_cs : int, dtmf_digits : int, barray : bytearray ):
        Qsk5_command.__init__(self, "send dtmf")
        
        self.play_len_cs = play_len_cs
        self.play_mute_cs = play_mute_cs
        self.dtmf_digits=dtmf_digits
        self.s_barray=barray

    # --------------------------------------------------
    def __str__(self):
        return self.command+' '+str(self.dtmf_digits)+' '+str(self.s_barray)

    # ---------------------------------------------------------------------------------------
    # NOTE that the byte_array is already given
    '''
    struct IPPU_senddtmf_req
       {
       struct IPPU_header16 header;  // can send a big amount of data !
       uint8_t  dtmf_play_cs;       // how long the dtmf should play in cs, eg 5
       uint8_t  dtmf_mute_cs;       // how long is the silence between tones ?
       uint16_t dtmf_digits;        // Needed since this is a generic send and the number of digits may be ODD
       uint8_t  byte_array[0];      // the actual data to send, the length is chunk_len -sizeof(struct Ipchunk_dtmf_req)
       };
    '''
    def send_request(self, serlib : Libuvk5serial ) -> Ippparser:
        
        # I could put the packing into the command.... bute there is no real added value...
        s_chunk = struct.pack('<BBH',self.play_len_cs,self.play_mute_cs,self.dtmf_digits)+self.s_barray
        
        buffer=serlib.ippp_send_chunk16(IPPU_senddtmf16_reqid,s_chunk)
        
        return Ippparser(buffer)
















# ===================================================================================
# Special request, mostly for debugging
class Qsk5_req_syscmd(Qsk5_command):

    '''
    struct IPPU_req_system_cmd
        {
        struct IPPU_header8 header;  // IPPU_specialcmd_req
        enum IPPU_SYSCMD_e sys_cmd;  // the actual command
        uint8_t  padding;            // free for use
        uint16_t w_power;            // the power level I wish, 0=low 1=mid 2,high
        uint16_t paddingb;           // free to use
        uint32_t w_freq_1x;          // the frequency to apply
        };
    '''
    # -------------------------------------------------------------------------------
    # standard constructor
    def __init__(self, w_cmd : int, w_p1=0, w_p2=0 ):
        Qsk5_command.__init__(self, "system command")
        
        self.w_cmd=w_cmd
        self.w_p1=w_p1
        self.w_p2=w_p2

    # --------------------------------------------------
    def __str__(self):
        return self.command

    # ---------------------------------------------------------------------------------------
    def send_request(self, serlib : Libuvk5serial ) -> Ippparser:
        
        s_chunk = struct.pack('<BBHHI',self.w_cmd,0,self.w_p1,0,self.w_p2)
        
        buffer=serlib.ippp_send_chunk(IPPU_systemcmd_reqid,s_chunk)
        
        return Ippparser(buffer)



# ===================================================================================
# Sending radio configuration
class Qsk5_req_set_radio_params(Qsk5_command):

    # -------------------------------------------------------------------------------
    # standard constructor
    def __init__(self, rxfreq_1x : int, power_bias : int, key_event : int ):
        Qsk5_command.__init__(self, "radio config")

        self.rxfreq_1x=rxfreq_1x
        self.power_bias=power_bias
        self.key_event = key_event;

    # --------------------------------------------------
    def __str__(self):
        return self.command+" rxf="+str(self.rxfreq_1x)+" pb="+str(self.power_bias)

    # ------------------------------------------------------
    # format and send a radio configuration
    '''
struct IPPU_req_radiocfg
   {
   struct IPPU_header8 header;   // IPPU_radiocfg_reqid
   uint16_t pad_even;
   uint32_t rx_freq_1x;       // receive on this f, zero means leave it alone, _1x means that you need to mult by 10
   uint32_t tx_freq_1x;       // send on this f, zero means leave it alone, can be > or < of rx
   uint8_t  tx_power;         // 0 means do not touch configuration, FF max power, intermediate numbers as appropriate
   uint8_t  select_chan_po;   // selected channel, PLUS ONE, so, 0 is a NULL value
   union IPPU_key_event_u key_event;   // if you wish to send a key event, write it here
   uint8_t  pad[1];
   };
    '''
    def send_request(self, serlib : Libuvk5serial ) -> Ippparser:
        
        s_chunk = struct.pack('<HIIBBBB',0,self.rxfreq_1x,self.rxfreq_1x,self.power_bias,0,self.key_event,0)
        
        buffer=serlib.ippp_send_chunk(IPPU_radiocfg_reqid,s_chunk)
        
        return Ippparser(buffer)
















# ===================================================================================
# this wraps a command to send a quansheng message
class Qsk5_req_send_qsmsg(Qsk5_command):

    # -------------------------------------------------------------------------------
    def __init__(self,to_address, s_msg : str ):
        Qsk5_command.__init__(self, "send qsmsg")
        
        self.to_address=to_address
        self.s_msg=s_msg

    # --------------------------------------------------
    def __str__(self):
        return self.command+' '+str(self.to_address)+' '+self.s_msg

    # ------------------------------------------------------
    # format and send a quansheng send message
    def send_request(self, serlib : Libuvk5serial ) -> Ippparser:
        
        #print('QS send try '+str(self.s_msg))
        
        ascii_bytes = bytearray(self.s_msg,'ascii', errors='replace')
        ascii_len = len(ascii_bytes)
        
        chunk_len = struct.calcsize('<HBBHBBBBBB')+ascii_len
        
        pack_len = chunk_len+2
        
        s_chunk = struct.pack('<HBBHBBBBBB',pack_len,0x20,chunk_len,1,0,self.to_address,0,1,ascii_len,0)+ascii_bytes
        buffer=serlib.ippp_send_chunk(IPPU_sendqsmsg_reqid,s_chunk)

        return Ippparser(buffer)

'''
# ===================================================================================
# this wraps a command to request a reboot
class Qsk5_req_reboot(Qsk5_command):

    # -------------------------------------------------------------------------------
    # standard constructor
    def __init__(self):
        Qsk5_command.__init__(self, Qsk5_command.cmd_qsk5_reboot)
        
    # ---------------------------------------------------------------------------------------
    # just the reboot, to be used at the end of EEPROM write
    # returns an empty IPP parser    
    def send_request(self, serlib : Libuvk5serial ) -> Ippparser:

        serlib.get_fw_version()
        serlib.QUANSHENG_reboot()

        return Ippparser(bytearray())
        
    # --------------------------------------------------
    # this is the equivalent of toString()
    def __str__(self):
        return self.command+'  NOW'
'''        
        
        
# ===================================================================================
# this wraps a command to request a spectrum scan
class Qsk5_req_spectrum_scan(Qsk5_command):

    # -------------------------------------------------------------------------------
    # standard constructor
    def __init__(self, start_F_1x : int, step_F_1x : int, step_count : int, stime_cs ):
        Qsk5_command.__init__(self, "spectrum scan")
        
        self.start_F_1x = start_F_1x
        self.step_F_1x = step_F_1x
        self.step_count = step_count
        self.stime_cs = stime_cs

    # --------------------------------------------------
    # this is the equivalent of toString()
    def __str__(self):
        return self.command+' '+str(self.start_F_1x)
        
    # ---------------------------------------------------------------------------------------
    '''
    struct IPPU_req_spectrum
       {
       struct IPPU_header8 header;  // IPPU_req_spectrum
       uint16_t samples_count;      // this is the number of steps (samples) to do
       uint32_t start_freq_1x;      // the starting frequency
       uint32_t step_freq_1x;       // the step can be HUGE, but should really not be
       uint16_t sample_time_cs;     // the time to take for a sample in cs, so you can do 100 samples per second, max
       uint16_t padding_even;       //
       };
    '''
    def send_request(self, serlib : Libuvk5serial ) -> Ippparser:

        s_chunk = struct.pack('<HIIHH',self.step_count,self.start_F_1x, self.step_F_1x, self.stime_cs, 0)
        
        buffer=serlib.ippp_send_chunk(IPPU_req_spectrum,s_chunk)
        
        return Ippparser(buffer)
        
        
        
        
        
        
# ===================================================================================
# this wraps a command to send a request identity
class Qsk5_req_identity(Qsk5_command):

    # -------------------------------------------------------------------------------
    # standard constructor
    def __init__(self):
        Qsk5_command.__init__(self, "req identity")

    # -------------------------------------------------------------------------------
    def send_request(self, serlib : Libuvk5serial ) -> Ippparser:
        payload = bytes()
        
        buffer=serlib.ippp_send_chunk(IPPU_ident_reqid,payload)
        return Ippparser(buffer)

    # --------------------------------------------------
    def __str__(self):
        return self.command





























