'''
/** Original work Copyright 2024 damiano IU3QGD
 * https://to-be-defined
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.
 */
'''
# -------------------------------------------------------------------------
# the idea is that the state of the app is shared between various threads
# since vars are somewhat local to a source file, you need to allocate it here and then share the "pointer"
# To avoid circular dependencies you need to avoid including in external modules this one
# It is kind of simple since you do not need to declare the type of a var...

from __future__ import annotations

from app_config import AppConfig
from app_gui import  InputParserThread, AppMainGui
from debug_gui import DbglsGui
from poll_gui import QpypollThread
from qsdtmf_gui import QsdtmfGui
from qseeprom_gui import QseepromGui
from qsk5_gui import NewAppGuiThread
from qsmsg_gui import QsmsgGui
from spectrum_gui import SpectrumGui


class Qpystat:
    
    def __init__(self):
        self.appconfig : AppConfig = None
        self.appguithread : NewAppGuiThread = None
        self.appmaingui : AppMainGui = None
        self.dbgls : DbglsGui = None
        self.inputParser : InputParserThread = None    
        self.qpypoll : QpypollThread = None
        self.qsmsg : QsmsgGui = None
        self.qsdtmf : QsdtmfGui = None
        self.qseeprom : QseepromGui = None
        self.spectrum : SpectrumGui = None

    # ---------------------------------------------------
    # You can use this one to find out if you should log something
    def mustLog(self, service, level ):
        if self.dbgls:
            return self.dbgls.mustLog(service, level)
        else:
            return True

    # ---------------------------------------------------
    # print something in the application log window
    def app_println(self, msg):
        if self.dbgls:
            self.dbgls.println(msg)
        else:
            print(msg)
    

    # --------------------------------------------------
    # this is the equivalent of toString()
    def __str__(self):
        return self.cwd
