'''
/** Original work Copyright 2024 damiano IU3QGD
 * https://to-be-defined
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.
 */
'''
# -------------------------------------------------------------------------
# There is only ONE poll, this is a GUI and if there is something to send it should be set to the poll

from __future__ import annotations

from tkinter import StringVar, IntVar

from app_config import AppConfig
from debug_gui import LogPanel
from glob_gui import TkToplevel, TV_Entry 
from ipp_parser import Qsk5_req_send_qsmsg
import tkinter.ttk as ttk


class QsmsgGui ():

    # ----------------------------------------------------------------
    def __init__(self, stat : 'Qpystat'):
        self.stat = stat
        
        self.stat.appconfig.addMyselfToSavables(self)

        self._toplevel = TkToplevel("Messenger window")
        
        aframe=ttk.Frame(self._toplevel, padding=2)     
    
        ttk.Label(aframe,text="Press ENTER to send message", font=("Courier", 24),anchor='center' ).pack(fill='x')
        
        self._newEditmsgPanel(aframe).pack(fill='x')
        
        self._qsmsg_log = LogPanel(self.stat,aframe,"Messages Log")
        self._qsmsg_log.setWrap(True);
        self._qsmsg_log.pack(fill='both',expand=True)
        
        aframe.pack(fill='both',expand=True)
        
        self._println("QsmsgGui init complete")

    # -------------------------------------------------------------------
    # prints something on the application log
    def _println(self, msg):
        self._qsmsg_log.println(msg)    
        
    # --------------------------------------------------------------------
    # show a window that has been hidden using the withdraw method
    def deiconify(self):
        self._toplevel.deiconify()    

    # --------------------------------------------------------------------
    # hide completely a window
    def withdraw(self):
        self._toplevel.withdraw()    

    # -----------------------------------------------------------
    # Mostly a reminder that this method is available everywhere    
    def _runOnGuiIdle(self, func, *args ):
        self.frame.after_idle(func, *args)


    # -------------------------------------------------------------------------------
    # make a new frame and put an input line in it
    def _newEditmsgPanel(self, parent):

        panel=ttk.Frame(parent,padding='3')

        # this variable will store whatever the user has typed
        self.edit_toid  = IntVar(panel)
        self.edit_qsmsg = StringVar(panel)
        
        ttk.Label(panel,text="To ID ").pack(side='left')
        TV_Entry(panel, self.edit_toid, width=4).pack(side='left')

        ttk.Label(panel,text="  Message ").pack(side='left')
        input_entry = TV_Entry(panel, self.edit_qsmsg)
        input_entry.pack(side='left', fill='x', expand=True)

        # if one press enter does the given action
        input_entry.bind('<Return>', self._sendMessageLineEnter)
        
        return panel
    
    
    # -------------------------------------------------------------------------------
    def _sendMessageLineEnter(self, _extra=None):

        try:
            anid = int(self.edit_toid.get())   
        except ValueError:
            self.edit_toid.set(33)
            anid = 33
    
        aline = self.edit_qsmsg.get()
        
        self.stat.app_println("to "+str(anid)+" seding "+aline)
        
        cmd = Qsk5_req_send_qsmsg(anid,aline)
        self.stat.qpypoll.queuePut(cmd)


    # -------------------------------------------------------------------------------
    # make a new frame and put buttons into it, but stay in this class
    def _newReadWriteButtonsPanel(self, container):
        cframe = ttk.Frame(container, borderwidth=3, relief='ridge')

        ttk.Button(cframe,text="ButtonA" , command=self._clicClients ).pack(side='left')
        ttk.Button(cframe,text="Help" , command=self._clicHelp).pack(side='left')

        return cframe
    
    # --------------------------------------------------------------------------------------
    def _clicClients(self):
        self.cmd_println("CLICK")

    # --------------------------------------------------------------------------------------
    def _clicHelp(self):
        self.cmd_println("Available commands")
    

    # -------------------------------------------------------------------
    # implement the config savable
    def appImportConfig(self, cfg : AppConfig ):
        adict = cfg.getDictValue("qsmsg_cfg", {})
        
        try:
            self._toplevel.setGeometry(adict['gui_cfg'])
            self.edit_toid.set(adict['edit_toid'])    
        except Exception as _exc :
            pass            

    # -------------------------------------------------------------------
    # implement the config savable
    def appSaveConfig(self, cfg : AppConfig ):
        adict = dict()

        adict['gui_cfg'] = self._toplevel.getGeometry()
        adict['edit_toid'] = self.edit_toid.get()

        cfg.setDictValue("qsmsg_cfg", adict )            
    
    

        
        

